@echo off

set ARG1=%~1
set ARG2=%~2
if "%ARG1%" equ "getciphername" goto :getciphername
if "%ARG1%" equ "gethashname"   goto :gethashname
if "%ARG1%" equ "getflavorname" goto :getflavorname
if "%ARG1%" equ "getpagingname" goto :getpagingname

verify other 2>nul
setlocal ENABLEEXTENSIONS
if errorlevel 1 goto no_extensions

rem +--------------------------------------------------------------------------+
rem | Default settings                                                         |
rem +--------------------------------------------------------------------------+

rem ** TrueCrypt drive letter (uppercase!) **
set DRIVE=T

rem ** TrueCrypt volume name (without spaces!) **
set VOLUME=%SystemDrive%\tctemp.tc

rem ** Image file name (without spaces!) **
set IMAGEFILE=%SystemDrive%\tctemp.img

rem ** TrueCrypt volume label **
set LABEL=Temp

rem ** Supported ciphers (uppercase!) **
rem A   = AES
rem B   = Blowfish
rem C   = CAST5
rem S   = Serpent
rem D   = Triple DES
rem T   = Twofish
rem AT  = AES-Twofish
rem ATS = AES-Twofish-Serpent
rem SA  = Serpent-AES
rem STA = Serpent-Twofish-AES
rem TS  = Twofish-Serpent
set CIPHER=A

rem ** Supported hash functions (lowercase!) **
rem r   = RIPEMD-160
rem s   = SHA-1
rem w   = WHIRLPOOL
set HASH=s

rem ** Supported flavors **
rem chk = debug version
rem fre = standard version
set FLAVOR=fre

rem ** Iteration Count Multiplier **
set ICMUL=1

rem ** Paging behavior **
rem def = system default
rem opt = optimized to reduce STOP errors
set PAGING=def

rem +--------------------------------------------------------------------------+
rem | Further settings                                                         |
rem +--------------------------------------------------------------------------+

rem ** Cache size for image file creation **
set IMAGECACHESIZE=10485760
rem set IMAGECACHESIZE=65536

rem ** TrueCrypt path (optional) **
set TCPATH=.

rem ** Configuration file creation **
rem 0   = don't save configuration
rem 1   = save configuration
rem 2   = save configuration and hide configuration file
SET SAVECONFIG=1

rem ** Configuration file name **
SET SETTINGS=cfg\settings.cmd

rem +--------------------------------------------------------------------------+
rem | Internal settings                                                        |
rem +--------------------------------------------------------------------------+

set VERSION=1.4a
set INSTALL=install.cmd
set DISMOUNTVOLUME=
set CMD=%ARG1%

if "%PAGING%" equ "opt" goto :paging_opt
set TRUECRYPT_DRIVER_START=default
set PAGING_EXECUTIVE=unchanged
set PAGING_KERNEL_STACKS=unchanged
goto :paging_end
:paging_opt
set TRUECRYPT_DRIVER_START=asap
set PAGING_EXECUTIVE=disabled
set PAGING_KERNEL_STACKS=disabled
:paging_end

if not exist %SETTINGS% goto :no_SETTINGSfile
call %SETTINGS% load
set /A ICMUL=%ICMUL%
:no_SETTINGSfile

rem +--------------------------------------------------------------------------+
rem | Location of executable files                                             |
rem +--------------------------------------------------------------------------+

if not exist "%TCPATH%\TrueCrypt.exe" goto :invalid_tcpath
if not exist "%TCPATH%\TrueCrypt Format.exe" goto :invalid_tcpath
set TCEXE="%TCPATH%\TrueCrypt.exe"
set TCFORMATEXE="%TCPATH%\TrueCrypt Format.exe"
goto :find_executables
:invalid_tcpath
set TCPATH=

:find_executables
set MISCEXE32=bin\misc.exe
set IMAGEEXE32=bin\image.exe
if not exist %MISCEXE32% goto no_miscexe32

%MISCEXE32% isadmin
if errorlevel 1 goto :no_admin
%MISCEXE32% iswin64
if not errorlevel 1 goto :x64

:x86
set TCDRIVER=system32\Drivers\truecrypt.sys
set BIN=bin
%MISCEXE32% osversion
if errorlevel 501 goto :x86_wxp
if not errorlevel 500 goto :unsupported_system
set OS=w2k
goto :x86_end

:x86_wxp
set OS=wxp

:x86_end
if "%TCPATH%" neq "" goto :xend
set TCEXE="%ProgramFiles%\TrueCrypt\TrueCrypt.exe"
set TCFORMATEXE="%ProgramFiles%\TrueCrypt\TrueCrypt Format.exe"
goto :xend

:x64
set TCDRIVER=SysWOW64\Drivers\truecrypt.sys
set BIN=bin-x64
set OS=wnet
if "%TCPATH%" neq "" goto :xend
set TCEXE="%ProgramFiles(x86)%\TrueCrypt\TrueCrypt.exe"
set TCFORMATEXE="%ProgramFiles(x86)%\TrueCrypt\TrueCrypt Format.exe"
goto :xend

:xend
set MISCEXE=%BIN%\misc.exe
if not exist %MISCEXE% goto no_miscexe

set TCTEMPEXE="%SystemRoot%\System32\tctemp.exe"
set TCTEMPEXE_XCOPY="%SystemRoot%\System32\?ctemp.exe"

rem +--------------------------------------------------------------------------+
rem | Main menu                                                                |
rem +--------------------------------------------------------------------------+

if "%CMD%" equ "install"     goto :install_main
if "%CMD%" equ "installmenu" goto :install
if "%CMD%" equ "createimage" goto :makeimg
if "%CMD%" equ "uninstall"   goto :uninstall

:main_menu
if not exist %TCTEMPEXE% goto :install

set TITLE=
call :cls
echo.
echo. (1) Re-install TCTEMP and update settings
echo. (2) Create image file
echo. (3) Uninstall TCTEMP
echo.
set input=
set /P input= Select a menu item (1-3): 
if "%INPUT%" equ "1" goto :install
if "%INPUT%" equ "2" goto :makeimg
if "%INPUT%" equ "3" goto :uninstall
if "%INPUT%" equ "" goto :eof
goto :main_menu

:return_to_main_menu
if not exist %TCTEMPEXE% goto :eof
goto :main_menu

rem +--------------------------------------------------------------------------+
rem | Install                                                                  |
rem +--------------------------------------------------------------------------+

:install
set TCTEMPBIN=%BIN%\tctemp-%OS%-%FLAVOR%.bin
if not exist %TCTEMPBIN% goto no_tctempbin
set RESULT=
call %INSTALL% getciphername "%CIPHER%"
set CIPHERNAME=%RESULT%
set RESULT=
call %INSTALL% gethashname "%HASH%"
set HASHNAME=%RESULT%
set RESULT=
call %INSTALL% getflavorname "%FLAVOR%"
set FLAVORNAME=%RESULT%
set RESULT=
call %INSTALL% getpagingname "%PAGING%"
set PAGINGNAME=%RESULT%
if "%CIPHERNAME%" equ "" goto invalid_cipher
if "%HASHNAME%" equ ""   goto invalid_hash
if "%FLAVORNAME%" equ "" goto invalid_flavor
if "%PAGINGNAME%" equ "" goto invalid_paging
set ICDEFAULT=1000
set /A IC=%ICMUL%*1000
if "%HASH%" equ "w" goto :install_menu
set ICDEFAULT=2000
set /A IC=%ICMUL%*2000
:install_updateic_end

:install_menu
set TITLE=
call :cls
echo. 
echo  Menu items:                 Installation settings:
echo.
echo. (1) Change cipher           Cipher         : %CIPHERNAME%
echo. (2) Change hash function    Hash function  : %HASHNAME%
echo. (3) Change iteration count  Iteration count: %ICMUL% * %ICDEFAULT% (= %IC%)
echo. (4) Change version type     Version type   : %FLAVORNAME%
echo. (5) Change volume name      Volume name    : %VOLUME%
echo. (6) Change image file       Image file     : %IMAGEFILE%
echo. (7) Change drive            Drive          : %DRIVE%:
echo. (8) Change volume label     Volume label   : %LABEL%
echo. (9) Change paging behavior  Paging behavior: %PAGINGNAME%
echo. (i) Install TCTEMP
echo.
set input=
set /P input= Select a menu item (1-9, i): 
if "%INPUT%" equ "1" goto :change_cipher
if "%INPUT%" equ "2" goto :change_hash
if "%INPUT%" equ "3" goto :change_iteration_count
if "%INPUT%" equ "4" goto :change_flavor
if "%INPUT%" equ "5" goto :change_volume
if "%INPUT%" equ "6" goto :change_imagefile
if "%INPUT%" equ "7" goto :change_drive
if "%INPUT%" equ "8" goto :change_label
if "%INPUT%" equ "9" goto :change_paging
if "%INPUT%" equ "i" goto :install_main
if "%INPUT%" equ "I" goto :install_main
if "%INPUT%" equ ""  goto :return_to_main_menu
goto :install_menu

:change_cipher
SET TITLE=Change cipher
call :cls
echo.
echo. Selected cipher: %CIPHERNAME%
echo.
echo.  (1) AES
echo.  (2) Blowfish
echo.  (3) Cast5
echo.  (4) Serpent
echo.  (5) Triple DES
echo.  (6) Twofish
echo.  (7) AES-Twofish
echo.  (8) AES-Twofish-Serpent
echo.  (9) Serpent-AES
echo. (10) Serpent-Twofish-AES
echo. (11) Twofish-Serpent
echo.
set input=
set /P input= Select a cipher: 
if "%INPUT%" equ "1"  goto select_cipher_1
if "%INPUT%" equ "2"  goto select_cipher_2
if "%INPUT%" equ "3"  goto select_cipher_3
if "%INPUT%" equ "4"  goto select_cipher_4
if "%INPUT%" equ "5"  goto select_cipher_5
if "%INPUT%" equ "6"  goto select_cipher_6
if "%INPUT%" equ "7"  goto select_cipher_7
if "%INPUT%" equ "8"  goto select_cipher_8
if "%INPUT%" equ "9"  goto select_cipher_9
if "%INPUT%" equ "10" goto select_cipher_10
if "%INPUT%" equ "11" goto select_cipher_11
goto :install
:select_cipher_1
set CIPHER=A
goto :install
:select_cipher_2
set CIPHER=B
goto :install
:select_cipher_3
set CIPHER=C
goto :install
:select_cipher_4
set CIPHER=S
goto :install
:select_cipher_5
set CIPHER=D
goto :install
:select_cipher_6
set CIPHER=T
goto :install
:select_cipher_7
set CIPHER=AT
goto :install
:select_cipher_8
set CIPHER=ATS
goto :install
:select_cipher_9
set CIPHER=SA
goto :install
:select_cipher_10
set CIPHER=STA
goto :install
:select_cipher_11
set CIPHER=TS
goto :install

:change_hash
SET TITLE=Change hash function
call :cls
echo.
echo. Selected hash function: %HASHNAME%
echo.
echo.  (1) RIPEMD-160
echo.  (2) SHA-1
echo.  (3) Whirlpool
echo.
set input=
set /P input= Select a hash function: 
if "%INPUT%" equ "1"  goto select_hash_1
if "%INPUT%" equ "2"  goto select_hash_2
if "%INPUT%" equ "3"  goto select_hash_3
goto :install
:select_hash_1
set HASH=r
goto :install
:select_hash_2
set HASH=s
goto :install
:select_hash_3
set HASH=w
goto :install

:change_iteration_count
SET TITLE=Change iteration count multiplier (for volume keys and password derivation)
call :cls
echo.
echo. Note that it is usually the best choice for most users to set the 
echo. iteration count multiplier to 1 (see chapter "Installing TCTEMP" of 
echo. TCTEMP.PDF for further information)
echo.
echo. Iteration count multiplier: %ICMUL%
echo. (Resulting iteration count: %IC%)
echo. 
set INPUT=
set /P INPUT= Enter iteration count multiplier (1-1000000): 
if "%INPUT%" equ "" goto :install
set /A INPUT=%INPUT%
if "%INPUT%" equ "0" goto :change_iteration_count
set INPUT_SIGN=%INPUT:~0,1%
if "%INPUT_SIGN%" equ "-" goto :change_iteration_count
set /A INPUT_EVAL=(%INPUT%-1)/1000000
if "%INPUT_EVAL%" neq "0" goto :change_iteration_count
set ICMUL=%INPUT%
goto :install

:change_flavor
SET TITLE=Change version type
call :cls
echo.
echo. Selected version type: %FLAVORNAME%
echo.
echo.  (1) Release
echo.  (2) Debug
echo.
set input=
set /P input= Select a hash function: 
if "%INPUT%" equ "1"  goto select_flavor_1
if "%INPUT%" equ "2"  goto select_flavor_2
goto :install
:select_flavor_1
set FLAVOR=fre
goto :install
:select_flavor_2
set FLAVOR=chk
goto :install

:change_drive
SET TITLE=Change drive letter
call :cls
echo.
echo. Selected drive: %DRIVE%:
echo.
set input=
set /P input= Enter drive letter: 
if "%INPUT%" equ "" goto :install
set INPUT=%INPUT:~0,1%
set INPUT=%INPUT:a=A%
set INPUT=%INPUT:b=B%
set INPUT=%INPUT:c=C%
set INPUT=%INPUT:d=D%
set INPUT=%INPUT:e=E%
set INPUT=%INPUT:f=F%
set INPUT=%INPUT:g=G%
set INPUT=%INPUT:h=H%
set INPUT=%INPUT:i=I%
set INPUT=%INPUT:j=J%
set INPUT=%INPUT:k=K%
set INPUT=%INPUT:l=L%
set INPUT=%INPUT:m=M%
set INPUT=%INPUT:n=N%
set INPUT=%INPUT:o=O%
set INPUT=%INPUT:p=P%
set INPUT=%INPUT:q=Q%
set INPUT=%INPUT:r=R%
set INPUT=%INPUT:s=S%
set INPUT=%INPUT:t=T%
set INPUT=%INPUT:u=U%
set INPUT=%INPUT:v=V%
set INPUT=%INPUT:w=W%
set INPUT=%INPUT:x=X%
set INPUT=%INPUT:y=Y%
set INPUT=%INPUT:z=Z%
if "%INPUT%" equ "A" goto :accept_drive_letter
if "%INPUT%" equ "B" goto :accept_drive_letter
if "%INPUT%" equ "C" goto :accept_drive_letter
if "%INPUT%" equ "D" goto :accept_drive_letter
if "%INPUT%" equ "E" goto :accept_drive_letter
if "%INPUT%" equ "F" goto :accept_drive_letter
if "%INPUT%" equ "G" goto :accept_drive_letter
if "%INPUT%" equ "H" goto :accept_drive_letter
if "%INPUT%" equ "I" goto :accept_drive_letter
if "%INPUT%" equ "J" goto :accept_drive_letter
if "%INPUT%" equ "K" goto :accept_drive_letter
if "%INPUT%" equ "L" goto :accept_drive_letter
if "%INPUT%" equ "M" goto :accept_drive_letter
if "%INPUT%" equ "N" goto :accept_drive_letter
if "%INPUT%" equ "O" goto :accept_drive_letter
if "%INPUT%" equ "P" goto :accept_drive_letter
if "%INPUT%" equ "Q" goto :accept_drive_letter
if "%INPUT%" equ "R" goto :accept_drive_letter
if "%INPUT%" equ "S" goto :accept_drive_letter
if "%INPUT%" equ "T" goto :accept_drive_letter
if "%INPUT%" equ "U" goto :accept_drive_letter
if "%INPUT%" equ "V" goto :accept_drive_letter
if "%INPUT%" equ "W" goto :accept_drive_letter
if "%INPUT%" equ "X" goto :accept_drive_letter
if "%INPUT%" equ "Y" goto :accept_drive_letter
if "%INPUT%" equ "Z" goto :accept_drive_letter
echo.
echo. Invalid drive letter!
echo.
pause
goto :change_drive
:accept_drive_letter
set DRIVE=%INPUT%
goto :install

:change_volume
SET TITLE=Change volume name
call :cls
echo.
echo. DEVICE-HOSTED VOLUMES: Please mind case sensitivity if you specify a 
echo. partition (e.g. \Device\Harddisk0\Partition2) instead of a container file. 
echo. Make also sure that you use correct values for the hard disk number and 
echo. for the partition number (TrueCrypt can be used to find the right values).
echo. 
echo. FILE-HOSTED VOLUMES: Please use only non-space ASCII characters.
echo.
echo.
echo.
echo. Current volume name  : %VOLUME%
echo.
set INPUT=
set /P INPUT= Enter new volume name: 
if "%INPUT%" equ "" goto :install
%MISCEXE% checkstrlen 259 "%INPUT%"
if errorlevel 1 goto :invalid_volume
%MISCEXE% checkstr nospace "%INPUT%"
if errorlevel 1 goto :invalid_volume
%MISCEXE% checkstr ascii "%INPUT%"
if errorlevel 1 goto :invalid_volume
%MISCEXE% checkstr tcfilename "%INPUT%"
if errorlevel 1 goto :invalid_volume
%MISCEXE% checkstr filename "%INPUT%"
if errorlevel 1 goto :valid_volume
%MISCEXE% findstr "%INPUT%" :
if errorlevel 1 goto :invalid_volume
:valid_volume
set VOLUME=%INPUT%
goto :install
:invalid_volume
echo.
echo. Invalid volume name!
echo.
pause
goto :change_volume

:change_imagefile
SET TITLE=Change image file name
call :cls
echo.
echo. Please use only non-space ASCII characters.
echo.
echo.
echo.
echo. Current image file name  : %IMAGEFILE%
echo.
set INPUT=
set /P INPUT= Enter new image file name: 
if "%INPUT%" equ "" goto :install
%MISCEXE% checkstrlen 259 "%INPUT%"
if errorlevel 1 goto :invalid_imagefile
%MISCEXE% checkstr nospace "%INPUT%"
if errorlevel 1 goto :invalid_imagefile
%MISCEXE% checkstr ascii "%INPUT%"
if errorlevel 1 goto :invalid_imagefile
%MISCEXE% checkstr filename "%INPUT%"
if errorlevel 1 goto :invalid_imagefile
%MISCEXE% findstr "%INPUT%" :
if errorlevel 1 goto :invalid_imagefile
set IMAGEFILE=%INPUT%
goto :install
:invalid_imagefile
echo.
echo. Invalid image file name!
echo.
pause
goto :change_imagefile

:change_label
SET TITLE=Change volume label
call :cls
echo.
echo. Current volume label  : %LABEL%
echo.
set INPUT=
set /P INPUT= Enter new volume label: 
if "%INPUT%" equ "" goto :install
set LABEL=%INPUT%
goto :install

:change_paging
SET TITLE=Change system's paging behavior
call :cls
echo.
echo. Note that it is known for the current TCTEMP version that a STOP error (blue
echo. screen) can occur during Windows shutdown after the paging file has been moved
echo. to the TCTEMP volume. This STOP error can either be completely avoided by not
echo. moving the paging file to the TCTEMP volume, or its occurance can be reduced 
echo. by changing the system's paging behavior (see chapter "Changing the Location 
echo. of the Paging File" of TCTEMP.PDF for further information).
echo.
echo. Note that it is recommended either to install TCGINA or to enable the
echo. TrueCrypt option "Auto-dismount all when user logs off" if the system's paging
echo. behavior has been optimized to reduce possible STOP errors.
echo.
echo. Selected paging behavior: %PAGINGNAME%
echo.
echo.  (1) System default
echo.  (2) Optimized to reduce STOP errors
echo.
set input=
set /P input= Select paging behavior: 
if "%INPUT%" equ "1"  goto select_paging_1
if "%INPUT%" equ "2"  goto select_paging_2
goto :install
:select_paging_1
set PAGING=def
goto :install
:select_paging_2
set PAGING=opt
goto :install

:install_main

call :reset_paging_behavior
if "%PAGING%" equ "opt" goto :paging2_opt
set TRUECRYPT_DRIVER_START=default
set PAGING_EXECUTIVE=unchanged
set PAGING_KERNEL_STACKS=unchanged
goto :paging2_end
:paging2_opt
set TRUECRYPT_DRIVER_START=asap
set PAGING_EXECUTIVE=disabled
set PAGING_KERNEL_STACKS=disabled
:paging2_end

if not exist "%SystemRoot%\%TCDRIVER%" goto :no_tcdriver
xcopy /y %TCTEMPBIN% %TCTEMPEXE_XCOPY% >nul
if errorlevel 1 goto :copy_error
if not exist %TCTEMPEXE% goto :copy_error
%MISCEXE% set REG_MULTI_SZ "HKLM\SYSTEM\CurrentControlSet\Control\Session Manager" BootExecute "autocheck autochk *&#0;tctemp %CIPHER% %HASH% %ICMUL% %DRIVE% %VOLUME% %IMAGEFILE%"
if errorlevel 1 goto :reg_error
if /i "%TRUECRYPT_DRIVER_START%" neq "asap" goto :install_main2
%MISCEXE% set REG_EXPAND_SZ HKLM\SYSTEM\CurrentControlSet\Services\truecrypt ImagePath "%TCDRIVER%"
if errorlevel 1 goto :reg_error
%MISCEXE% set REG_DWORD HKLM\SYSTEM\CurrentControlSet\Services\truecrypt Start 0
if errorlevel 1 goto :reg_error
%MISCEXE% set REG_DWORD HKLM\SYSTEM\CurrentControlSet\Services\truecrypt Tag 1
if errorlevel 1 goto :reg_error
%MISCEXE% set REG_SZ HKLM\SYSTEM\CurrentControlSet\Services\truecrypt Group "Boot Bus Extender"
if errorlevel 1 goto :reg_error

:install_main2
%MISCEXE% protectfile "%SystemRoot%\System32\tctemp.exe"
if errorlevel 1 goto :protect_tctempexe_error
if /i "%PAGING_EXECUTIVE%" neq "disabled" goto :install_main3
%MISCEXE% set REG_DWORD "HKLM\SYSTEM\CurrentControlSet\Control\Session Manager\Memory Management" DisablePagingExecutive 1
if errorlevel 1 goto :reg_error

:install_main3
if /i "%PAGING_KERNEL_STACKS%" neq "disabled" goto :install_main4
rem gflags -r +dps >nul
rem if not errorlevel 1 goto :install_main4
%MISCEXE% set REG_DWORD "HKLM\SYSTEM\CurrentControlSet\Control\Session Manager" GlobalFlag 0x80000
if errorlevel 1 goto :reg_error

:install_main4
if %SAVECONFIG% equ "0" goto :install_end
attrib -h %SETTINGS%
echo.@echo off>%SETTINGS%
echo.rem ** Configuration file (created by TCTEMP V%VERSION% Setup) **>>%SETTINGS%
echo.if "%%1" neq "load" goto :syntax_error>>%SETTINGS%
echo.set CIPHER=%CIPHER%>>%SETTINGS%
echo.set HASH=%HASH%>>%SETTINGS%
echo.set FLAVOR=%FLAVOR%>>%SETTINGS%
echo.set ICMUL=%ICMUL% >>%SETTINGS%
echo.set DRIVE=%DRIVE%>>%SETTINGS%
echo.set VOLUME=%VOLUME%>>%SETTINGS%
echo.set LABEL=%LABEL%>>%SETTINGS%
echo.set IMAGEFILE=%IMAGEFILE%>>%SETTINGS%
echo.set PAGING=%PAGING%>>%SETTINGS%
echo.set TRUECRYPT_DRIVER_START=%TRUECRYPT_DRIVER_START%>>%SETTINGS%
echo.set PAGING_EXECUTIVE=%PAGING_EXECUTIVE%>>%SETTINGS%
echo.set PAGING_KERNEL_STACKS=%PAGING_KERNEL_STACKS%>>%SETTINGS%
echo.set CMD=%CMD%>>%SETTINGS%
echo.goto :eof>>%SETTINGS%
echo.>>%SETTINGS%
echo.:syntax_error>>%SETTINGS%
echo.cls>>%SETTINGS%
echo.echo.This is a configuration file for TCTEMP Setup>>%SETTINGS%
echo.echo.>>%SETTINGS%
echo.pause>>%SETTINGS%
if "%SAVECONFIG%" equ "1" goto :install_end
attrib +h %SETTINGS%

:install_end
set ENDMESSAGE=TCTEMP has been successfully installed.
if not exist %IMAGEFILE% goto :install_makeimg
if "%VOLUME:~0,16%" equ "\Device\Harddisk" goto :end
if not exist %VOLUME% goto :install_makeimg
goto :end

:install_makeimg
SET TITLE=
call :cls
echo.
echo. %ENDMESSAGE%
echo.
echo. The TrueCrypt volume and the image file need to be created.
echo.
pause

rem +--------------------------------------------------------------------------+
rem | Create image file                                                        |
rem +--------------------------------------------------------------------------+

:makeimg
if not exist %IMAGEEXE32% goto no_imageexe32
if not exist %TCEXE% goto no_tcexe
if not exist %TCFORMATEXE% goto no_tcformatexe
if not exist %TCTEMPEXE% goto :no_tctempexe

%MISCEXE% drivetype %DRIVE%
if errorlevel 2 goto :volume_already_mounted
if not exist %TCEXE% goto no_tcexe
if not exist %TCFORMATEXE% goto no_tcformatexe

set TITLE=TrueCrypt volume and image file creation
call :cls
echo.
echo. TrueCrypt volume: %VOLUME%
echo. Image file      : %IMAGEFILE%
echo. Drive           : %DRIVE%: (will be used temporarily to mount the TrueCrypt volume)
echo.
set input=
set /P input= Do you want to create the TrueCrypt volume and the image file (y/n)? 
if "%input%" neq "y" goto :main_menu

call :cls
echo.
echo. Create a TrueCrypt volume for TCTEMP and use the following volume name:
echo.
echo.     %VOLUME%
echo.
%TCFORMATEXE%
%TCEXE% /q /s /f /d %DRIVE%
call :cls
echo.
echo. Enter the volume password...
%TCEXE% /a /q /l %DRIVE% /v "%VOLUME%" /c y
%MISCEXE% drivetype %DRIVE%
if not errorlevel 2 goto :volume_not_mounted
set DISMOUNTVOLUME=1

:fill
call :cls
echo.
echo. Operation in progress: Filling TrueCrypt volume with a recognizable pattern

rem ** The following lines are no longer required because image.exe should now
rem ** be able to access the last sectors of a volume even if the volume is formatted
rem ** with NTFS
rem %IMAGEEXE32% -c%IMAGECACHESIZE% f %DRIVE%: 0 1
rem if errorlevel 1 goto :fill_error
rem %TCEXE% /q /s /f /d %DRIVE%
rem %TCEXE% /a /q /l %DRIVE% /v "%VOLUME%"

%MISCEXE% print " "
%IMAGEEXE32% -c%IMAGECACHESIZE% -d78 f %DRIVE%:
if errorlevel 1 goto :fill_error

rem ** The following lines are no longer required because image.exe now properly
rem ** locks/unlocks the raw volume during its operation
rem %TCEXE% /q /s /f /d %DRIVE%
rem %TCEXE% /a /q /l %DRIVE% /v "%VOLUME%"
rem echo.
rem echo.
rem echo. (Please wait 3 seconds)
rem call :sleep 3000

:format_method
call :cls
echo.
echo. Please select a method to quick format drive %DRIVE%:
echo.
echo. (1) Quick format drive %DRIVE%: with NTFS
echo. (2) Quick format drive %DRIVE%: with FAT
echo. (3) Quick format drive %DRIVE%: with FAT32
echo. (4) Use explorer to quick format drive %DRIVE%:
echo. (0) Abort TCTEMP Setup
echo.
set input=
set /P input= Method for quick formatting (0-4): 
if "%INPUT%" equ "0" goto :format_exit
if "%INPUT%" equ "1" goto :format_ntfs
if "%INPUT%" equ "2" goto :format_fat32
if "%INPUT%" equ "3" goto :format_fat
if "%INPUT%" equ "4" goto :format_explorer
goto format_method

:format_ntfs
format %DRIVE%: /q /v:%LABEL% /fs:NTFS
if errorlevel 1 goto :format_error
goto format_end

:format_fat32
format %DRIVE%: /q /v:%LABEL% /fs:FAT
if errorlevel 1 goto :format_error
goto format_end

:format_fat
format %DRIVE%: /q /v:%LABEL% /fs:FAT32
if errorlevel 1 goto :format_error
goto format_end

:format_explorer
explorer /e,%DRIVE%:
echo.
echo. Press any key when done.
echo.
pause
dir %DRIVE%:\ | find "%DRIVE%:" >nul
if errorlevel 1 goto :format_error
goto :format_end

:format_exit
%TCEXE% /q /s /f /d %DRIVE%
goto :eof

:format_error
echo.
echo. ERROR: Cannot quick format drive %DRIVE%: with selected file system
echo. Please select another method to quick format drive %DRIVE%:
echo.
pause
goto :fill


:format_end
call :cls
echo.
set input=
set /P input= Do you want to create a folder which can be used by the print spooler (y/n)? 
if "%input%" neq "y" goto :format_end2
md %DRIVE%:\spool\PRINTERS

:format_end2
echo.
echo. You can optionally copy files/folders to drive %DRIVE%: and change its volume label.
echo.
echo. Press any key when done to start image file creation.
echo.
pause

call :cls
echo.
echo. Operation in progress: Creating image file

rem ** remount the volume to invalidate all open handles
%TCEXE% /q /s /f /d %DRIVE%
%TCEXE% /a /q /l %DRIVE% /v "%VOLUME%"

rem ** The following lines are no longer required because image.exe should now
rem ** be able to access the last sectors of a volume even if the volume is formatted
rem ** with NTFS
rem %IMAGEEXE32% -c%IMAGECACHESIZE% c %DRIVE%: "%IMAGEFILE%" 0 1
rem if errorlevel 1 goto :image_error
rem %IMAGEEXE32% -c%IMAGECACHESIZE% f %DRIVE%: 0 1
rem if errorlevel 1 goto :image_error
rem %TCEXE% /q /s /f /d %DRIVE%
rem %TCEXE% /a /q /l %DRIVE% /v "%VOLUME%"

%MISCEXE% print " "
rem %IMAGEEXE32% -c%IMAGECACHESIZE% -d78 u %DRIVE%: "%IMAGEFILE%" 
%IMAGEEXE32% -c%IMAGECACHESIZE% -d78 c %DRIVE%: "%IMAGEFILE%" 
if errorlevel 1 goto :image_error

rem ** the following lines are not required
rem call :cls
rem echo.
rem echo. Operation in progress: Restoring file system
rem echo.
rem %IMAGEEXE32% r %DRIVE%: "%IMAGEFILE%"
rem if errorlevel 1 goto :image_error

%TCEXE% /q /s /f /d %DRIVE% /w /c n
%MISCEXE% protectsystemfile "%IMAGEFILE%"
if errorlevel 1 goto :protect_imagefile_error
if "%VOLUME:~0,16%" equ "\Device\Harddisk" goto :image_end
%MISCEXE% protectsystemfile "%VOLUME%"
if errorlevel 1 goto :protect_volume_error

:image_end
set ENDMESSAGE=The TrueCrypt volume and image file have been successfully created.
goto :end

rem +--------------------------------------------------------------------------+
rem | Reset paging behavior to system default                                  |
rem +--------------------------------------------------------------------------+

:reset_paging_behavior
if not exist "%SystemRoot%\%TCDRIVER%" goto :reset_paging_behavior2
%MISCEXE% set REG_DWORD HKLM\SYSTEM\CurrentControlSet\Services\truecrypt Start 1
%MISCEXE% set REG_EXPAND_SZ HKLM\SYSTEM\CurrentControlSet\Services\truecrypt ImagePath "\??\%SystemRoot%\%TCDRIVER%"
%MISCEXE% delete HKLM\SYSTEM\CurrentControlSet\Services\truecrypt Group
%MISCEXE% delete HKLM\SYSTEM\CurrentControlSet\Services\truecrypt Tag
:reset_paging_behavior2
%MISCEXE% delete "HKLM\SYSTEM\CurrentControlSet\Control\Session Manager\Memory Management" DisablePagingExecutive
%MISCEXE% delete "HKLM\SYSTEM\CurrentControlSet\Control\Session Manager" GlobalFlag
goto :eof

rem +--------------------------------------------------------------------------+
rem | Uninstall                                                                |
rem +--------------------------------------------------------------------------+

:uninstall
call :reset_paging_behavior
del %TCTEMPEXE%
%MISCEXE% set REG_MULTI_SZ "HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\Session Manager" BootExecute "autocheck autochk *"
if errorlevel 1 goto reg_error
set ENDMESSAGE=TCTEMP has been successfully uninstalled.
goto :end

rem +--------------------------------------------------------------------------+
rem | Query functions                                                          |
rem +--------------------------------------------------------------------------+

:getciphername
if "%ARG2%" equ "A"   goto :cipher_A
if "%ARG2%" equ "B"   goto :cipher_B
if "%ARG2%" equ "C"   goto :cipher_C
if "%ARG2%" equ "D"   goto :cipher_D
if "%ARG2%" equ "S"   goto :cipher_S
if "%ARG2%" equ "T"   goto :cipher_T
if "%ARG2%" equ "AT"  goto :cipher_AT
if "%ARG2%" equ "ATS" goto :cipher_ATS
if "%ARG2%" equ "SA"  goto :cipher_SA
if "%ARG2%" equ "STA" goto :cipher_STA
if "%ARG2%" equ "TS"  goto :cipher_TS
goto :eof
:cipher_A
set Result=AES
goto :eof
:cipher_B
set Result=Blowfish
goto :eof
:cipher_C
set Result=Cast5
goto :eof
:cipher_D
set Result=Triple DES
goto :eof
:cipher_S
set Result=Serpent
goto :eof
:cipher_T
set Result=Twofish
goto :eof
:cipher_AT
set Result=AES-Twofish
goto :eof
:cipher_ATS
set Result=AES-Twofish-Serpent
goto :eof
:cipher_SA
set Result=Serpent-AES
goto :eof
:cipher_STA
set Result=Serpent-Twofish-AES
goto :eof
:cipher_TS
set Result=Twofish-Serpent
goto :eof

:gethashname
if "%ARG2%" equ "r" goto :hash_r
if "%ARG2%" equ "s" goto :hash_s
if "%ARG2%" equ "w" goto :hash_w
goto :eof
:hash_r
set Result=RIPEMD-160
goto :eof
:hash_s
set Result=SHA-1
goto :eof
:hash_w
set Result=Whirlpool
goto :eof

:getflavorname
if "%ARG2%" equ "chk" goto :flavor_chk
if "%ARG2%" equ "fre" goto :flavor_fre
goto :eof
:flavor_chk
set Result=Debug
goto :eof
:flavor_fre
set Result=Release
goto :eof

:getpagingname
if "%ARG2%" equ "def" goto :paging_def
if "%ARG2%" equ "opt" goto :paging_opt
goto :eof
:paging_def
set Result=System default
goto :eof
:paging_opt
set Result=Optimized to reduce STOP errors
goto :eof

rem +--------------------------------------------------------------------------+
rem | Auxiliary functions                                                      |
rem +--------------------------------------------------------------------------+

:cls
cls
echo. ------------------------------------------------------------------------------
echo. TCTEMP V%VERSION% Setup
if "%TITLE%" equ "" goto :cls_no_title
echo. %TITLE%
:cls_no_title
echo. ------------------------------------------------------------------------------
goto :eof

:sleep
rem set /A SEC=(%1+1500)/1000 >nul
rem ping -n %SEC% 127.0.0.1 >nul
%MISCEXE% sleep %1
goto :eof

:end
set TITLE=
call :cls
echo.
echo. %ENDMESSAGE%
echo.
pause
goto :eof

rem +--------------------------------------------------------------------------+
rem | Error messages                                                           |
rem +--------------------------------------------------------------------------+

:error
if "%DISMOUNTVOLUME%" equ "" goto :error2
%TCEXE% /q /s /f /d %DRIVE%
:error2
set TITLE=
call :cls
echo.
echo. ERROR: %ERROR%
echo.
pause
goto :eof

:no_extensions
set ERROR=Unsupported command file processor!
goto :error

:no_miscexe32
set ERROR=Cannot find "%MISCEXE32%"!
goto :error

:no_imageexe32
set ERROR=Cannot find "%IMAGEEXE32%"!
goto :error

:no_miscexe
set ERROR=Cannot find "%MISCEXE%"!
goto :error

:no_tctempbin
set ERROR=Cannot find "%TCTEMPBIN%"!
goto :error

:no_tctempexe
set ERROR=Cannot find %TCTEMPEXE%!
goto :error

:no_tcexe
set ERROR=Cannot find "TrueCrypt.exe"!
goto :error

:no_tcformatexe
set ERROR=Cannot find "TrueCrypt Format.exe"!
goto :error

:no_tcdriver
set ERROR=Cannot find "truecrypt.sys" (TrueCrypt is possibly not installed)!
goto :error

:invalid_cipher
set ERROR=Unsupported cipher (id=%CIPHER%)!
goto :error

:invalid_hash
set ERROR=Unsupported hash function (id=%HASH%)!
goto :error

:invalid_flavor
set ERROR=Unsupported flavor (%FLAVOR%)!
goto :error

:invalid_paging
set ERROR=Unsupported paging behavior (%PAGING%)!
goto :error

:no_admin
set ERROR=Administrator privileges required!
goto :error

:unsupported_system
set ERROR=Operating system not supported!
goto :error

:copy_error
set ERROR=Cannot copy TCTEMP.EXE!
goto :error

:del_error
set ERROR=Cannot delete TCTEMP.EXE!

:reg_error
set ERROR=Cannot change registry value!
goto :error

:protect_tctempexe_error
set ERROR=Cannot revoke access rights to %TCTEMPEXE%!
goto :error

:protect_imagefile_error
set ERROR=Cannot revoke access rights to %IMAGEFILE%!
goto :error

:protect_volume_error
set ERROR=Cannot revoke access rights to %VOLUME%!
goto :error

:volume_already_mounted
set ERROR=The TCTEMP drive %DRIVE%: is in use!
goto :error

:volume_not_mounted
set ERROR=The TrueCrypt volume is not mounted!
goto :error

:fill_error
set ERROR=Cannot fill TrueCrypt volume with a recognizable pattern (ErrorCode = %errorlevel%)!
goto :error

:image_error
set ERROR=Cannot create image file!
goto :error

